% linreg.m
clear; clc;
nsim = 20000; burnin = 1000; 

    % generate the data
T = 500; % sample size
beta = [1 5]';  sig2 = .5;
X = [ones(T,1) 1+randn(T,1)];
y = X*beta + sqrt(sig2)*randn(T,1);

    % prior
beta0 = [0 0]'; iVbeta0 = eye(2)/100;
nu0 = 3;  S0 = 1*(nu0-1);

    % initialize the Markov chain
beta = (X'*X)\(X'*y);  
sig2 = sum((y-X*beta).^2)/T;
store_theta = zeros(nsim,3);

for isim = 1:nsim + burnin
        % sample beta
    Dbeta = (iVbeta0 + X'*X/sig2)\speye(2); % same as inv
    beta_hat = Dbeta*(iVbeta0*beta0 + X'*y/sig2);
    C = chol(Dbeta,'lower');
    beta = beta_hat + C*randn(2,1);

        % sample sig2
    e = y-X*beta;
    sig2 = 1/gamrnd(nu0 + T/2,1/(S0 + e'*e/2));

        % store the parameters
    if isim > burnin
        isave = isim - burnin;        
        store_theta(isave,:) = [beta' sig2];    
    end
end
theta_hat = mean(store_theta)
theta_CI = quantile(store_theta,[.025 .975])
