clear; clc;
nsim = 50000; burnin = 1000;

% load data
data_raw = load('USPCE_2015Q4.csv');
data = 400*log(data_raw(2:end)./data_raw(1:end-1));
y0 = data(1:2); % [y_{-1}, y_0]
y = data(3:end);
T = size(y,1);
X = [ones(T,1) [y0(2); y(1:end-1)] [y0; y(1:end-2)]];

% prior
beta0 = zeros(3,1); iVbeta = speye(3)/100;
nu0 = 3; S0 = 1*(nu0 - 1);

% initialize the Markov chain
psi = 0;
beta = (X'*X)\(X'*y);
sig2 = sum((y-X*beta).^2)/T;
Hpsi = speye(T) + sparse(2:T,1:(T-1),psi*ones(1,T-1),T,T); 
store_theta = zeros(nsim,5);  % [beta' sig2 psi]
store_dev = zeros(nsim,1);    % deviance
count_psi = 0;

for isim = 1:nsim + burnin
        % sample beta    
    X_tilde = Hpsi\X; y_tilde = Hpsi\y;    
    Dbeta = (iVbeta + X_tilde'*X_tilde/sig2)\speye(3); 
    beta_hat = Dbeta*(iVbeta*beta0 + X_tilde'*y_tilde/sig2);    
    beta = beta_hat + chol(Dbeta,'lower')*randn(3,1);
    
        % sample psi    
    e = y - X*beta;
    [psi, flag] = sample_psi(psi,e,sig2);    
    Hpsi = speye(T) + sparse(2:T,1:(T-1),psi*ones(1,T-1),T,T); 
    count_psi = count_psi + flag;
   
        % sample sig2
    tmp = Hpsi\e;
    sig2 = 1/gamrnd(nu0+T/2,1/(S0 + tmp'*tmp/2)); 
    
    if (mod(isim, 5000) == 0)
        disp([num2str(isim) ' loops... ']);
    end
    
        % store the parameters
    if isim > burnin
        isave = isim - burnin;
        store_theta(isave,:) = [beta' sig2 psi];    
        
        % for DIC computation
        store_dev(isave) = -2*llike_MA1(psi,y-X*beta,sig2);        
    end
end
theta_hat = mean(store_theta)
theta_CI = quantile(store_theta,[.025 .975])

% plot histogram of psi
hist(store_theta(:,end),50); box off;