function ml = linreg_ma1_chib(store_theta,y,X,beta0,iVbeta,nu0,S0)
nsim_re = 5000; % size of reduced runs
[nsim,m] = size(store_theta);
T = size(y,1);
theta_hat = mean(store_theta)';
beta_s = theta_hat(1:m-2);
sig2_s = theta_hat(m-1);
psi_s = theta_hat(m);

    % evaluate the log likelihood at beta_s,sig2_s,psi_s
llike = llike_MA1(psi_s,y-X*beta_s,sig2_s);

    % define the prior
prior = @(b,s,p) lmvnpdf(b,beta0,iVbeta\speye(3)) + ligampdf(s,nu0,S0) ...
    + log(1/2);

    % evaluate the posterior of beta at beta_s
store_lpbeta = zeros(nsim,1);
for isim = 1:nsim
    sig2 = store_theta(isim,m-1);
    psi = store_theta(isim,m);
    Hpsi = speye(T) + sparse(2:T,1:(T-1),psi*ones(1,T-1),T,T);     
    X_tilde = Hpsi\X; y_tilde = Hpsi\y;    
    Dbeta = (iVbeta + X_tilde'*X_tilde/sig2)\speye(3); 
    beta_hat = Dbeta*(iVbeta*beta0 + X_tilde'*y_tilde/sig2);    
    store_lpbeta(isim) = lmvnpdf(beta_s,beta_hat,Dbeta); 
end
lpbeta = log(mean(exp(store_lpbeta)));

    % evaluate the posterior of sig2 at sig2_s using a reduced run
beta = beta_s; % fix beta at the posterior mean
e = y - X*beta;
sig2 = sig2_s;
store_lpsig2 = zeros(nsim_re,1);
for isim = 1:nsim_re
        % sample psi
    psi = sample_psi(psi,e,sig2); 
    Hpsi = speye(T) + sparse(2:T,1:(T-1),psi*ones(1,T-1),T,T);    
        % sample sig2
    tmp = Hpsi\e;
    sig2 = 1/gamrnd(nu0+T/2,1/(S0 + tmp'*tmp/2)); 
    
    store_lpsig2(isim,:) = ligampdf(sig2_s,nu0+T/2,S0 + tmp'*tmp/2);
end
lpsig2 = log(mean(exp(store_lpsig2)));

    % evaluate the posterior of psi at psi_s
n_grid = 799; % number of grid points
psi_grid = linspace(-.99,.99,n_grid)';  % build a grid for psi
psi_grid = sort([psi_grid;psi_s]);  % insert psi_s
n_grid = size(psi_grid,1);
idx_psi = (psi_grid==psi_s); % index for psi_s
lp_psi = zeros(n_grid,1);  % log posterior density
for igrid = 1:n_grid
    psi = psi_grid(igrid);
    lp_psi(igrid) = llike_MA1(psi,y-X*beta_s,sig2_s);
end    
p_psi = exp(lp_psi-max(lp_psi));  % exponentiate the log-density
p_psi = p_psi/(sum(p_psi)*(psi_grid(2)-psi_grid(1))); % normalize
lppsi = log(p_psi(idx_psi));
ml = llike + prior(beta_s,sig2_s,psi_s) - (lpbeta + lpsig2 + lppsi);
end