% linreg_t.m
clear; clc;
nsim = 20000; burnin = 1000;

    % load data
data_raw = load('USPCE_2015Q4.csv');
data = 400*log(data_raw(2:end)./data_raw(1:end-1));
y0 = data(1:2); % [y_{-1}, y_0]
y = data(3:end);
T = size(y,1);
X = [ones(T,1) [y0(1); y(1:end-1)] [y0; y(1:end-2)]];

    % prior
beta0 = zeros(3,1); iVbeta = speye(3)/100;
rho0 = 0; iVrho = 1;
nu0 = 3; S0 = 1*(nu0 - 1);
nu_ub = 50;

    % initialize the Markov chain
nu = 5;
beta = (X'*X)\(X'*y);
sig2 = sum((y-X*beta).^2)/T;
lam = 1./gamrnd(nu/2,2/nu,T,1);
iLam = sparse(1:T,1:T,1./lam);

store_theta = zeros(nsim,5);  % [beta' sig2 nu]
count_nu = 0;

for isim = 1:nsim + burnin
        % sample beta    
    Dbeta = (iVbeta + X'*iLam*X/sig2)\speye(3); 
    beta_hat = Dbeta*(iVbeta*beta0 + X'*iLam*y/sig2);
    C = chol(Dbeta,'lower');
    beta = beta_hat + C*randn(3,1);
    
        % sample lam
    e = y - X*beta;
    lam = 1./gamrnd((nu+1)/2,2./(nu+e.^2/sig2));
    iLam = sparse(1:T,1:T,1./lam);
    
        % sample sig2    
    sig2 = 1/gamrnd(nu0+T/2,1/(S0 + e'*iLam*e/2)); 
    
        % sample nu
    [nu,flag] = sample_nu_MH(lam,nu,nu_ub);
    count_nu = count_nu + flag;
    
        % store the parameters
    if isim > burnin
        isave = isim - burnin;
        store_theta(isave,:) = [beta' sig2 nu];      
    end
end
theta_hat = mean(store_theta)
theta_CI = quantile(store_theta,[.025 .975])