function [ml,ml_std] = linreg_t_CE(store_theta,y,X,prior,R)
R = 20*ceil(R/20);   % make R divisible by 20
m = size(store_theta,2);
T = size(y,1);
    % obtain parameters for the IS density
b_hat = mean(store_theta(:,1:m-2))';
B_hat = cov(store_theta(:,1:m-2))';
tmp = gamfit(1./store_theta(:,m-1));
gam1_hat = tmp(1); gam2_hat = 1./tmp(2);
tmp = gamfit(1./store_theta(:,m));
alp1_hat = tmp(1); alp2_hat = 1./tmp(2);

    % obtain IS draws from the optimal density
theta_IS = zeros(R,m);
theta_IS(:,1:m-2) = repmat(b_hat',R,1) + (chol(B_hat,'lower')*randn(m-2,R))'; 
theta_IS(:,m-1) = 1./gamrnd(gam1_hat,1./gam2_hat,R,1);
theta_IS(:,m) = 1./gamrnd(alp1_hat,1./alp2_hat,R,1);

    % construct the IS density
g_IS = @(b,s,n) lmvnpdf(b,b_hat,B_hat) + ligampdf(s,gam1_hat,gam2_hat) ...
    + ligampdf(n,alp1_hat,alp2_hat);
store_w = zeros(R,1);

for isim  = 1:R
    theta = theta_IS(isim,:)';
    beta = theta(1:m-2);
    sig2 = theta(m-1);
    nu = theta(m);
    llike = T*(gammaln((nu+1)/2) - gammaln(nu/2) - .5*log(nu*pi*sig2)) ...
        - (nu+1)/2*sum(log(1 + (y-X*beta).^2/(sig2*nu)));
    store_w(isim) = llike + prior(beta,sig2,nu) - g_IS(beta,sig2,nu);
end
shortw = reshape(store_w,R/20,20); % divide the draws into 20 batches
maxw = max(shortw); % find the max for normalization

bigml = log(mean(exp(shortw-repmat(maxw,R/20,1)),1)) + maxw;
ml = mean(bigml);
ml_std = std(bigml)/sqrt(20);
end